library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.numeric_std.ALL;

entity Lauflicht_if is port (	
		-- common
		clk      : in std_logic;
		-- LEDs
		reset, dir, faster, slower : in std_logic;
		led      : out std_logic_vector (6 downto 0);
		bulp     : out std_logic;
		
		-- 7-Segments
      xy_sel	: in std_logic;
      sseg_an  : out std_logic_vector(3 downto 0);
      sseg_ca 	: out std_logic;
      sseg_cb 	: out std_logic;
      sseg_cc 	: out std_logic;
      sseg_cd 	: out std_logic;
      sseg_ce 	: out std_logic;
      sseg_cf 	: out std_logic;
      sseg_cg 	: out std_logic
		);
end Lauflicht_if;

architecture Lauflicht_Impl of Lauflicht_if is
   component led_if is
      Port ( 
			l_clk      : in std_logic;
			l_reset    : in std_logic;
			l_dir      : in std_logic;
			l_faster   : in std_logic;
			l_slower   : in std_logic;
			l_led      : out std_logic_vector (6 downto 0);
			l_bulp     : out std_logic);
   end component;
   
   component segment_if is
      Port (
			l_clk      : in std_logic;
			l_xy_sel	  : in std_logic;
			l_sseg_an  : out std_logic_vector(3 downto 0);
			l_sseg_ca  : out std_logic;
			l_sseg_cb  : out std_logic;
			l_sseg_cc  : out std_logic;
			l_sseg_cd  : out std_logic;
			l_sseg_ce  : out std_logic;
			l_sseg_cf  : out std_logic;
			l_sseg_cg  : out std_logic);
   end component;

begin
	led_i : led_if
   port map( l_clk => clk,
             l_reset => reset,
				 l_dir => dir,
				 l_faster => faster,
				 l_slower => slower,
				 l_led => led,
				 l_bulp => bulp);
	
   segment_i : segment_if
   port map( l_clk => clk,
             l_xy_sel => xy_sel,
				 l_sseg_an => sseg_an,
				 l_sseg_ca => sseg_ca,
				 l_sseg_cb => sseg_cb,
				 l_sseg_cc => sseg_cc,
				 l_sseg_cd => sseg_cd,
				 l_sseg_ce => sseg_ce,
				 l_sseg_cf => sseg_cf,
				 l_sseg_cg => sseg_cg);
	
end Lauflicht_Impl;

