library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity led_if is port (	
		-- common
		l_clk    : in std_logic; 
		-- LEDs
		l_reset  : in std_logic;
		l_dir    : in std_logic;
		l_faster : in std_logic;
		l_slower : in std_logic;
		l_led    : out std_logic_vector (6 downto 0);
		l_bulp   : out std_logic
		);
end led_if;


architecture led_impl of led_if 
is
	-- =====
	-- INPUT
	-- =====

	-- clock-in helper for buttons
	signal insr_reset : std_logic_vector(1 downto 0);
	signal insr_dir   : std_logic_vector(2 downto 0);
	signal insr_faster: std_logic_vector(2 downto 0);
	signal insr_slower: std_logic_vector(2 downto 0);
	
	-- reset button
	signal reset_reg  : std_logic;
	
	-- dir button rules direction and a bulp (LED)
	signal dir_reg    : std_logic;
	signal bulp_reg   : std_logic;
	
	-- buttons 'faster' and 'slower' rule speed
	signal speed_reg  : unsigned (7 downto 0) := "10000000";
	
	-- timer tick
	signal count_reg  : unsigned (31 downto 0);
	signal count_next : unsigned (31 downto 0) := (others => '0');

	-- ======
	-- OUTPUT
	-- ======
	
	-- led
	signal led_reg    : unsigned (6 downto 0) := "0000001";
	signal led_next   : unsigned (6 downto 0);
	
begin
	
	-- clock-in buttons
	process
	begin
		wait until rising_edge(l_clk);
		
		insr_reset  <= insr_reset(0)           & l_reset;
		insr_dir    <= insr_dir(1 downto 0)    & l_dir;
		insr_faster <= insr_faster(1 downto 0) & l_faster;
		insr_slower <= insr_slower(1 downto 0) & l_slower;
		
	end process;
	
	-- set the registers
	process 
	begin
		wait until rising_edge(l_clk);
		
		count_reg <= count_next;
		led_reg <= led_next;
		
		-- reset
		if (insr_reset(1) = '1') then
			reset_reg <= '1';
		else
			reset_reg <= '0';
		end if;
		
		-- direction
		if (insr_dir(2 downto 1) = "01") then 
			dir_reg <= not dir_reg;
			bulp_reg <= not bulp_reg;
		end if;

		-- faster
		if (insr_faster(2 downto 1) = "01") then 
			if speed_reg(0) = '1'
			then
				speed_reg <= "10000000";
			else 
				speed_reg <= '0' & speed_reg(7 downto 1);
			end if;
		end if;
		
		-- slower
		if (insr_slower(2 downto 1) = "01") then 
			if speed_reg(7) = '1'
			then
				speed_reg <= "00000001"; 
			else
				speed_reg <= speed_reg(6 downto 0) & '0';
			end if;
		end if;
		
	end process;
 
	-- calculate the next values for the registers
	process 
	begin
		wait until rising_edge(l_clk);
		
		if (reset_reg = '1') then
			if (dir_reg = '0') then
				led_next <= "0000001";
			else
				led_next <= "1000000";
			end if;	
			count_next <= (others => '0');
		elsif (count_reg >= speed_reg & "0000000000000000") then
		   if (dir_reg = '0') then
				led_next <= led_reg(5 downto 0) & led_reg(6);
			else
		    	led_next <= led_reg(0) & led_reg(6 downto 1);
			end if;
			count_next <= (others => '0');
		else
			led_next <= led_reg;
			count_next <= count_reg + 1;
		end if;
		
	end process;

	-- output
	l_led <= std_logic_vector(led_reg);
	l_bulp <= bulp_reg;
	
end led_impl;