library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity segment_if is port (
		-- common
		l_clk      : in std_logic;
		-- 7-Segments
      l_xy_sel	  : in std_logic;
      l_sseg_an  : out std_logic_vector(3 downto 0);
      l_sseg_ca  : out std_logic;
      l_sseg_cb  : out std_logic;
      l_sseg_cc  : out std_logic;
      l_sseg_cd  : out std_logic;
      l_sseg_ce  : out std_logic;
      l_sseg_cf  : out std_logic;
      l_sseg_cg  : out std_logic
		);
end segment_if;

architecture segment_impl of segment_if is

	signal sseg_anode      : unsigned(1 downto 0);
	signal sseg_anode_data : std_logic_vector(3 downto 0);
	signal sseg_cathode    : std_logic_vector(6 downto 0);
	signal sseg_data       : std_logic_vector(3 downto 0);
	
	signal x_data 			  : std_logic_vector(15 downto 0);
	signal y_data 			  : std_logic_vector(15 downto 0);
	signal xy_data 		  : std_logic_vector(15 downto 0);
	
begin	
	
	x_data <= x"4321";  -- test data x
	y_data <= x"8765";  -- test data y
	
	xy_data <= x_data when l_xy_sel = '0' else y_data;
	
	clk_prescaler: process (l_clk)
		variable cnt : integer range 0 to 10000000; -- 20,000 ~= 1.67ms x 4 = refresh
	begin
		if l_clk'event and l_clk = '1' then
			if cnt = 10000000 then -- counter prescaler
				cnt := 0;
			else
				cnt := cnt + 1;
			end if;
			if cnt = 10000000 then -- sseg anode sel
				sseg_anode <= sseg_anode + 1;
			end if;
		end if;
	end process;
	
	sseg_enable: process (sseg_anode, xy_data)
	begin
		case sseg_anode is
			when "00" => -- disp 1
				sseg_anode_data <= "1110";          -- first sseg is turned off
				sseg_data <= xy_data(15 downto 12); -- sseg_cath_data
			when "01" => -- disp 2
				sseg_anode_data <= "1101";
				sseg_data <= xy_data(11 downto 8);
			when "10" => -- disp 3
				sseg_anode_data <= "1011";
				sseg_data <= xy_data(7 downto 4);
			when "11" => -- disp 4
				sseg_anode_data <= "0111";
				sseg_data <= xy_data(3 downto 0);
			when others =>
				sseg_data <= "1111";
		end case;
	end process;
	
	sseg_decode: process (sseg_data)
	begin
		case sseg_data is                            -- "abcdefg"
			when "0000" => sseg_cathode <= "0000001"; -- '0'
			when "0001" => sseg_cathode <= "1001111"; -- '1'
			when "0010" => sseg_cathode <= "0010010"; -- '2'
			when "0011" => sseg_cathode <= "0000110"; -- '3'
			when "0100" => sseg_cathode <= "1001100"; -- '4'
			when "0101" => sseg_cathode <= "0100100"; -- '5'
			when "0110" => sseg_cathode <= "0100000"; -- '6'
			when "0111" => sseg_cathode <= "0001111"; -- '7'
			when "1000" => sseg_cathode <= "0000000"; -- '8'
			when "1001" => sseg_cathode <= "0000100"; -- '9'
			when "1010" => sseg_cathode <= "0001000"; -- 'A'
			when "1011" => sseg_cathode <= "1100000"; -- 'b'
			when "1100" => sseg_cathode <= "0110001"; -- 'C'
			when "1101" => sseg_cathode <= "1000010"; -- 'd'
			when "1110" => sseg_cathode <= "0110000"; -- 'E'
			when "1111" => sseg_cathode <= "0111000"; -- 'F'
			when others => sseg_cathode <= "1111111"; -- 'off'
		end case;
	end process;
	
	-- output
	l_sseg_an <= sseg_anode_data;
	
	l_sseg_ca <= sseg_cathode(6);
	l_sseg_cb <= sseg_cathode(5);
	l_sseg_cc <= sseg_cathode(4);
	l_sseg_cd <= sseg_cathode(3);
	l_sseg_ce <= sseg_cathode(2);
	l_sseg_cf <= sseg_cathode(1);
	l_sseg_cg <= sseg_cathode(0);

end segment_impl; 

